Package: ggh4x
Title: Hacks for 'ggplot2'
Version: 0.2.8
Authors@R: 
    person(given = "Teun",
           family = "van den Brand",
           role = c("aut", "cre"),
           email = "tahvdbrand@gmail.com",
           comment = c(ORCID = "0000-0002-9335-7468"))
Description: A 'ggplot2' extension that does a variety of little
    helpful things.  The package extends 'ggplot2' facets through
    customisation, by setting individual scales per panel, resizing panels
    and providing nested facets.  Also allows multiple colour and fill
    scales per plot. Also hosts a smaller collection of stats, geoms and axis 
    guides.
License: MIT + file LICENSE
URL: https://github.com/teunbrand/ggh4x,
        https://teunbrand.github.io/ggh4x/
BugReports: https://github.com/teunbrand/ggh4x/issues
Depends: ggplot2 (>= 3.4.2)
Imports: grid, gtable, scales, vctrs (>= 0.5.0), rlang (>= 1.1.0),
        lifecycle, stats, cli
Suggests: covr, fitdistrplus, ggdendro, vdiffr, knitr, MASS, rmarkdown,
        testthat (>= 3.0.0), utils
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.0
Config/testthat/edition: 3
Collate: 'borrowed_ggplot2.R' 'conveniences.R' 'ggh4x_extensions.R'
        'coord_axes_inside.R' 'element_part_rect.R' 'facet_grid2.R'
        'facet_wrap2.R' 'facet_manual.R' 'facet_nested.R'
        'facet_nested_wrap.R' 'facetted_pos_scales.R'
        'force_panelsize.R' 'geom_box.R' 'geom_outline_point.R'
        'geom_pointpath.R' 'geom_polygonraster.R' 'geom_rectrug.R'
        'geom_text_aimed.R' 'ggh4x-package.R' 'guide_axis_logticks.R'
        'guide_axis_manual.R' 'guide_axis_minor.R'
        'guide_axis_nested.R' 'guide_axis_scalebar.R'
        'guide_axis_truncated.R' 'guide_axis_utils.R'
        'guide_dendrogram.R' 'guide_stringlegend.R' 'help_secondary.R'
        'position_disjoint_ranges.R' 'position_lineartrans.R'
        'scale_dendrogram.R' 'scale_facet.R' 'scale_listed.R'
        'scale_manual.R' 'scale_multi.R' 'stat_difference.R'
        'stat_funxy.R' 'stat_rle.R' 'stat_roll.R' 'stat_theodensity.R'
        'strip_vanilla.R' 'strip_themed.R' 'strip_nested.R'
        'strip_split.R' 'themes.R' 'utils.R' 'utils_grid.R'
        'utils_gtable.R'
NeedsCompilation: no
Packaged: 2024-01-23 20:26:28 UTC; Teun
Author: Teun van den Brand [aut, cre] (<https://orcid.org/0000-0002-9335-7468>)
Maintainer: Teun van den Brand <tahvdbrand@gmail.com>
Repository: CRAN
Date/Publication: 2024-01-23 21:00:02 UTC
Built: R 4.4.0; ; 2024-05-30 03:25:49 UTC; unix
