# scale_listed throws error if scalelist and replaces unequal length

    The `replaces` argument must be parallel to and of the same length as the `scalelist` argument.

# scale_listed throws error when replaces has invalid aes

    The aesthetics in the `replaces` argument must be valid aesthetics.

# scale_listed throws error when non-scales are supplied as scalelist

    The `scalelist` argument must have valid <Scale> objects as list-elements.

# scale_listed throws error when multiple aesthetics are supplied in a scale

    `scale_listed()` can only accept 1 aesthetic per scale.

# scale_lsited throws error when empty aesthetics are supplied in a scale

    Every scale in the `scalelist` argument must have set valid aesthetics.

